@@ -1,4 +1,4 @@
-# Copyright (C) 2018 Shaon Ghosh, Xiaoshu Liu
+# Copyright (C) 2022 Shaon Ghosh, Michael Camilo, Xiaoshu Liu
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -76,42 +76,6 @@ def get_LambdaT_for_eos(m1, m2, max_mass_eos, eosfunc):
     return LambdaT
 
 
-def join_json_files(list_of_jsons, nametag="model"):
-    '''
-    This helper function joins the JSON files from the output of the Bayes
-    factor computation methods in Model_selection and Stacking into one single
-    JSON file that can be directly used for further analysis.
-
-    list_of_jsons :: A list of all the JSON files that needs to be combined.
-    nametag :: Use this option to create a name-tag for the resulting JSON
-    files
-    '''
-    alldata = []
-    reference_models = []
-    for json_file in list_of_jsons:
-        with open(json_file, 'r') as f:
-            thisdata = json.load(f)
-            alldata.append(thisdata)
-            reference_models.append(thisdata['ref_eos'])
-
-    # Keep unique names of the equation of state models
-    reference_models = np.unique(np.array(reference_models)).tolist()
-
-    for model in reference_models:
-        combined_dict = {}
-        for data in alldata:
-            if model == data['ref_eos']:
-                value = {'joint_bf': data['joint_bf'],
-                         'joint_bf_array': data['joint_bf_array'],
-                         'all_bf': data['all_bf'],
-                         'all_bf_err': data['all_bf_err']}
-                combined_dict[data['target_eos']] = value
-
-        filename = 'bayes_factors_against_' + model + '_' + nametag + '.json'
-        with open(filename, 'w') as f:
-            json.dump(combined_dict, f, indent=2, sort_keys=True)
-
-
 # The integrator function #
 def integrator(q_min, q_max, mc, eosfunc, max_mass_eos, postfunc,
                gridN=1000, var_LambdaT=1.0, var_q=1.0, minMass=0.1):
@@ -235,7 +199,7 @@ def get_trials(fd):
 
 
 class Model_selection:
-    def __init__(self, posteriorFile, priorFile=None):
+    def __init__(self, posteriorFile, priorFile=None, spectral=False):
         '''
         Initiates the Bayes factor calculator with the posterior
         samples from the uniform LambdaT, dLambdaT parameter
@@ -250,7 +214,11 @@ class Model_selection:
                          obtained from the prior file. If this is not
                          supplied, the posterior samples will be used
                          to determine the bounds.
+
+        spectral      :: Distinguishes between piecewise polytrope and spectral 
+                         decomposition method.
         '''
+
         self.data = np.recfromtxt(posteriorFile, names=True)
 
         if priorFile:
@@ -289,6 +257,9 @@ class Model_selection:
                                   yhigh=self.yhigh,
                                   bw=self.bw)
 
+        # Attribute that distinguishes parametrization method
+        self.spectral = spectral
+
     def getEoSInterp(self, eosname=None, m_min=1.0, N=100):
         '''
         This method accepts one of the NS native equations of state
@@ -344,6 +315,7 @@ class Model_selection:
         Lambdas = np.array(Lambdas)
         gravMass = np.array(gravMass)
         s = interp1d(gravMass, Lambdas)
+
         return [s, gravMass, Lambdas, max_mass]
 
     def getEoSInterpFromMLambdaFile(self, tidalFile):
@@ -401,8 +373,54 @@ class Model_selection:
         max_mass = np.max(masses)
         return [s, masses, Lambdas, max_mass]
 
-    def computeEvidenceRatio(self, EoS1, EoS2, gridN=1000,
-                             save=None, trials=0, verbose=False):
+    def getEoSInterp_parametrized(self, params, N=100):
+        '''
+        This method accepts a four parameter description of the neutron star 
+        equation of state, and returns a list [s, m_min, max_mass] where s is 
+        the interpolation function for the mass and the tidal deformability.
+
+        params      :: Four parameter list.
+
+        N           :: Number of points that will be used for the
+                       construction of the interpolant.
+        '''
+
+        if self.spectral == False:
+            log_p1_SI, g1, g2, g3 = params
+            eos = lalsim.SimNeutronStarEOS4ParameterPiecewisePolytrope(log_p1_SI, g1, g2, g3)
+        else:
+            g0, g1, g2, g3 = params
+            eos = lalsim.SimNeutronStarEOS4ParameterSpectralDecomposition(g0, g1, g2, g3)
+
+        fam = lalsim.CreateSimNeutronStarFamily(eos)
+        m_min = lalsim.SimNeutronStarFamMinimumMass(fam)/lal.MSUN_SI
+        max_mass = lalsim.SimNeutronStarMaximumMass(fam)/lal.MSUN_SI
+
+        # This is necessary so that interpolant is computed over the full range
+        # Keeping number upto 3 decimal places
+        # Not rounding up, since that will lead to RuntimeError
+        max_mass = int(max_mass*1000)/1000
+        masses = np.linspace(m_min, max_mass, N)
+        masses = masses[masses <= max_mass]
+        Lambdas = []
+        gravMass = []
+        for m in masses:
+            try:
+                rr = lalsim.SimNeutronStarRadius(m*lal.MSUN_SI, fam)
+                kk = lalsim.SimNeutronStarLoveNumberK2(m*lal.MSUN_SI, fam)
+                cc = m*lal.MRSUN_SI/rr
+                Lambdas = np.append(Lambdas, (2/3)*kk/(cc**5))
+                gravMass = np.append(gravMass, m)
+            except RuntimeError:
+                break
+        Lambdas = np.array(Lambdas)
+        gravMass = np.array(gravMass)
+        s = interp1d(gravMass, Lambdas)
+        
+        return([s, m_min, max_mass])
+
+    def computeEvidenceRatio(self, EoS1, EoS2, gridN=1000, save=None, 
+                             trials=0, verbose=False):
         '''
         This method computes the ratio of evidences for two
         tabulated EoS. It first checks if a file exists with
@@ -426,7 +444,11 @@ class Model_selection:
 
         # generate interpolators for both EOS
 
-        if os.path.exists(EoS1):
+        if type(EoS1) == list:
+            [s1, _,
+             max_mass_eos1] = self.getEoSInterp_parametrized(EoS1, N=1000)
+
+        elif os.path.exists(EoS1):
             if verbose:
                 print('Trying m-R-k file to compute EoS interpolant')
             try:
@@ -442,7 +464,11 @@ class Model_selection:
              max_mass_eos1] = self.getEoSInterp(eosname=EoS1,
                                                 m_min=self.minMass)
 
-        if os.path.exists(EoS2):
+        if type(EoS2) == list:
+            [s2, _,
+             max_mass_eos2] = self.getEoSInterp_parametrized(EoS2, N=1000)
+
+        elif os.path.exists(EoS2):
             if verbose:
                 print('Trying m-R-k file to compute EoS interpolant')
             try:
@@ -536,7 +562,29 @@ class Model_selection:
         ray.shutdown()
         return [support2D1/support2D2, sup_array]
 
+    def eos_evidence(self, params, gridN=1000):
+        '''
+        This method computes the evidence for a parametrized EoS.
 
+        params      :: Four parameter list.
+        gridN       :: Number of grid points over which the
+                       line-integral is computed. (Default = 1000)
+        '''
+
+        # generate interpolator for eos
+        [s, _,
+         max_mass_eos] = self.getEoSInterp_parametrized(params, N=1000)
+
+        # compute support
+        [lambdat_eos,
+         q_eos, support2D] = integrator(self.q_min, self.q_max, self.mc_mean,
+                                        s, max_mass_eos, self.kde,
+                                        gridN=gridN,
+                                        var_LambdaT=self.var_LambdaT,
+                                        var_q=self.var_q,
+                                        minMass=self.minMass)
+
+        return(support2D)
 
     def plot_func(self, eos_list, gridN=1000, filename='posterior_support.pdf',
                   full_mc_dist=False, usetitle=False):
@@ -562,6 +610,7 @@ class Model_selection:
                         values of the chirp mass.
         usetitle :: List of EoS on the title of the plot (Default: False)
         '''
+
         import pylab as pl
 
         pl.clf()
@@ -614,7 +663,14 @@ class Model_selection:
         if type(eos_list) == str:
             eos_list = [eos_list]
         for eos in eos_list:
-            if os.path.exists(eos):
+            if type(eos) == list:
+                [s, _,
+                 max_mass_eos] = self.getEoSInterp_parametrized(eos, N=1000)
+
+                # Reducing the text in the figure legend
+                eos = [np.round(eos[0], 4), np.round(eos[1], 4), np.round(eos[2], 4),np.round(eos[3], 4)]
+
+            elif os.path.exists(eos):
                 print('Trying m-R-k file to compute EoS interpolant')
                 try:
                     [s, _, _,
@@ -709,7 +765,8 @@ class Stacking():
             print('Number of prior and posterior files should be same')
             sys.exit(0)
 
-    def stack_events(self, EoS1, EoS2, trials=0, gridN=1000, save=None, verbose=False):
+    def stack_events(self, EoS1, EoS2, trials=0, gridN=1000, save=None, 
+                     verbose=False):
         '''
         Loop through each event and compute the joint Bayes-factor.
         Each individual event's Bayes-factor can be accessed from the Stacking
@@ -779,6 +836,7 @@ class Stacking():
                                                        gridN=gridN,
                                                        trials=trials,
                                                        verbose=verbose)
+
             if type(bayes_factor) == np.float64:
                 joint_bf *= bayes_factor
                 self.all_bayes_factors.append(bayes_factor)
@@ -822,6 +880,31 @@ class Stacking():
 
         return joint_bf
 
+    def joint_evidence(self, EoS, gridN=1000):
+        '''
+        Loop through each event and compute the joint evidence. Each individual 
+        event's evidence can be accessed from the Stacking object, using 
+        Stacking.all_evidences.
+
+        EoS :: The list of parameters that characterise the equation of state. 
+               This can be in the form of either one of the two supported 
+               parametrized equations of state: Spectral Decomposition and 
+               Piecewise Polytrope.
+
+        gridN :: Number of grid points over which the line-integral is
+                 computed (Default = 1000).
+        '''
+        joint_evidence = 1.0
+        self.all_evidences = []  # To be populated by B.Fs from all events
+
+        for prior_file, event_file in zip(self.event_priors, self.event_list):
+            modsel = Model_selection(posteriorFile=event_file,
+                                     priorFile=prior_file)
+
+            joint_evidence *= modsel.eos_evidence(EoS, gridN=gridN)
+
+        return joint_evidence
+
     def plot_stacked_bf(self, eos_list=None, ref_eos='SLY', trials=0,
                         gridN=1000, filename='stacked_bf.pdf'):
         '''
@@ -870,6 +953,7 @@ class Stacking():
         for eos in eos_list:
             print('Stacking events for model: {}'.format(eos))
             this_eos_bf = self.stack_events(eos, ref_eos, trials=trials)
+
             if trials > 0:
                 bf_combined.append(this_eos_bf[0])
                 d_bf_combined.append(2*np.std(this_eos_bf[-1]))
